@@ -4,6 +4,7 @@ import math
 import os
 import os.path as osp
 import re
+import sys
 import webbrowser
 
 import imgviz
@@ -17,13 +18,14 @@ from PyQt5.QtWidgets import (
     QPlainTextEdit,
     QVBoxLayout,
     QWhatsThis,
+    QMessageBox,
 )
 
 from anylabeling.services.auto_labeling.types import AutoLabelingMode
 
 from ...app_info import __appname__
 from . import utils
-from .config import get_config
+from .config import get_config, save_config
 from .label_file import LabelFile, LabelFileError
 from .logger import logger
 from .shape import Shape
@@ -47,10 +49,11 @@ LABEL_COLORMAP[2] = LABEL_COLORMAP[1]
 LABEL_COLORMAP[1] = [0, 180, 33]
 
 
-class LabelmeWidget(LabelDialog):
-    """Labelme widget"""
+class LabelingWidget(LabelDialog):
+    """The main widget for labeling images"""
 
     FIT_WINDOW, FIT_WIDTH, MANUAL_ZOOM = 0, 1, 2
+    next_files_changed = QtCore.pyqtSignal(list)
 
     def __init__(
         self,
@@ -267,7 +270,7 @@ class LabelmeWidget(LabelDialog):
             self.open_next_image,
             shortcuts["open_next"],
             "next",
-            self.tr("Open next (hold Ctl+Shift to copy labels)"),
+            self.tr("Open next (hold Ctrl+Shift to copy labels)"),
             enabled=False,
         )
         open_prev_image = action(
@@ -275,7 +278,7 @@ class LabelmeWidget(LabelDialog):
             self.open_prev_image,
             shortcuts["open_prev"],
             "prev",
-            self.tr("Open prev (hold Ctl+Shift to copy labels)"),
+            self.tr("Open prev (hold Ctrl+Shift to copy labels)"),
             enabled=False,
         )
         save = action(
@@ -323,19 +326,20 @@ class LabelmeWidget(LabelDialog):
         save_auto.setChecked(self._config["auto_save"])
 
         save_with_image_data = action(
-            text="Save With Image Data",
+            text=self.tr("Save With Image Data"),
             slot=self.enable_save_image_with_data,
-            tip="Save image data in label file",
+            icon="save",
+            tip=self.tr("Save image data in label file"),
             checkable=True,
             checked=self._config["store_data"],
         )
 
         close = action(
-            "&Close",
+            self.tr("&Close"),
             self.close_file,
             shortcuts["close"],
             "cancel",
-            "Close current file",
+            self.tr("Close current file"),
         )
 
         toggle_keep_prev_mode = action(
@@ -343,11 +347,23 @@ class LabelmeWidget(LabelDialog):
             self.toggle_keep_prev_mode,
             shortcuts["toggle_keep_prev_mode"],
             None,
-            self.tr('Toggle "keep pevious annotation" mode'),
+            self.tr('Toggle "Keep Previous Annotation" mode'),
             checkable=True,
         )
         toggle_keep_prev_mode.setChecked(self._config["keep_prev"])
 
+        toggle_auto_use_last_label_mode = action(
+            self.tr("Auto Use Last Label"),
+            self.toggle_auto_use_last_label,
+            None,
+            None,
+            self.tr('Toggle "Auto Use Last Label" mode'),
+            checkable=True,
+        )
+        toggle_auto_use_last_label_mode.setChecked(
+            self._config["auto_use_last_label"]
+        )
+
         create_mode = action(
             self.tr("Create Polygons"),
             lambda: self.toggle_draw_mode(False, create_mode="polygon"),
@@ -462,11 +478,11 @@ class LabelmeWidget(LabelDialog):
             enabled=False,
         )
         remove_point = action(
-            text="Remove Selected Point",
+            text=self.tr("Remove Selected Point"),
             slot=self.remove_selected_point,
             shortcut=shortcuts["remove_selected_point"],
             icon="edit",
-            tip="Remove selected point from polygon",
+            tip=self.tr("Remove selected point from polygon"),
             enabled=False,
         )
 
@@ -494,6 +510,13 @@ class LabelmeWidget(LabelDialog):
             enabled=False,
         )
 
+        documentation = action(
+            self.tr("&Documentation"),
+            self.documentation,
+            icon="help",
+            tip=self.tr("Show documentation"),
+        )
+
         contact = action(
             self.tr("&Contact me"),
             self.contact,
@@ -569,7 +592,7 @@ class LabelmeWidget(LabelDialog):
             enabled=False,
         )
         brightness_contrast = action(
-            "&Brightness Contrast",
+            self.tr("&Brightness Contrast"),
             self.brightness_contrast,
             None,
             "color",
@@ -604,6 +627,32 @@ class LabelmeWidget(LabelDialog):
             enabled=True,
         )
 
+        # Languages
+        select_lang_en = action(
+            "English",
+            functools.partial(self.set_language, "en_US"),
+            icon="us",
+            checkable=True,
+            checked=self._config["language"] == "en_US",
+            enabled=self._config["language"] != "en_US",
+        )
+        select_lang_vi = action(
+            "Tiếng Việt",
+            functools.partial(self.set_language, "vi_VN"),
+            icon="vn",
+            checkable=True,
+            checked=self._config["language"] == "vi_VN",
+            enabled=self._config["language"] != "vi_VN",
+        )
+        select_lang_zh = action(
+            "中文",
+            functools.partial(self.set_language, "zh_CN"),
+            icon="cn",
+            checkable=True,
+            checked=self._config["language"] == "zh_CN",
+            enabled=self._config["language"] != "zh_CN",
+        )
+
         # Group zoom controls into a list for easier toggling.
         zoom_actions = (
             self.zoom_widget,
@@ -670,6 +719,7 @@ class LabelmeWidget(LabelDialog):
             close=close,
             delete_file=delete_file,
             toggle_keep_prev_mode=toggle_keep_prev_mode,
+            toggle_auto_use_last_label_mode=toggle_auto_use_last_label_mode,
             delete=delete,
             edit=edit,
             duplicate=duplicate,
@@ -713,6 +763,7 @@ class LabelmeWidget(LabelDialog):
                 remove_point,
                 None,
                 toggle_keep_prev_mode,
+                toggle_auto_use_last_label_mode,
             ),
             # menu shown at right click
             menu=(
@@ -756,6 +807,7 @@ class LabelmeWidget(LabelDialog):
             file=self.menu(self.tr("&File")),
             edit=self.menu(self.tr("&Edit")),
             view=self.menu(self.tr("&View")),
+            language=self.menu(self.tr("&Language")),
             help=self.menu(self.tr("&Help")),
             recent_files=QtWidgets.QMenu(self.tr("Open &Recent")),
             label_list=label_menu,
@@ -779,7 +831,21 @@ class LabelmeWidget(LabelDialog):
                 None,
             ),
         )
-        utils.add_actions(self.menus.help, (contact,))
+        utils.add_actions(
+            self.menus.help,
+            (
+                documentation,
+                contact,
+            ),
+        )
+        utils.add_actions(
+            self.menus.language,
+            (
+                select_lang_en,
+                select_lang_vi,
+                select_lang_zh,
+            ),
+        )
         utils.add_actions(
             self.menus.view,
             (
@@ -839,12 +905,8 @@ class LabelmeWidget(LabelDialog):
             self.actions.create_point_mode,
             self.actions.create_line_strip_mode,
             edit_mode,
-            # duplicate,
-            # copy,
-            # paste,
             delete,
             undo,
-            # brightness_contrast,
             None,
             zoom,
             fit_width,
@@ -878,6 +940,18 @@ class LabelmeWidget(LabelDialog):
         self.auto_labeling_widget.finish_auto_labeling_object_action_requested.connect(
             self.finish_auto_labeling_object
         )
+        self.auto_labeling_widget.model_manager.prediction_started.connect(
+            lambda: self.canvas.set_loading(True, self.tr("Please wait..."))
+        )
+        self.auto_labeling_widget.model_manager.prediction_finished.connect(
+            lambda: self.canvas.set_loading(False)
+        )
+        self.next_files_changed.connect(
+            self.auto_labeling_widget.model_manager.on_next_files_changed
+        )
+        self.auto_labeling_widget.model_manager.request_next_files_requested.connect(
+            lambda: self.inform_next_files(self.filename)
+        )
         self.auto_labeling_widget.hide()  # Hide by default
         central_layout.addWidget(self.label_instruction)
         central_layout.addWidget(self.auto_labeling_widget)
@@ -997,11 +1071,34 @@ class LabelmeWidget(LabelDialog):
         if self.first_start:
             QWhatsThis.enterWhatsThisMode()
 
+        self.set_text_editing(False)
+
+    def set_language(self, language):
+        if self._config["language"] == language:
+            return
+        self._config["language"] = language
+        save_config(self._config)
+
+        # Show dialog to restart application
+        msg_box = QMessageBox()
+        msg_box.setText(
+            self.tr("Please restart the application to apply changes.")
+        )
+        msg_box.exec_()
+        self.parent.parent.close()
+
     def get_labeling_instruction(self):
+        text_mode = self.tr("Mode:")
+        text_shortcuts = self.tr("Shortcuts:")
+        text_previous = self.tr("Previous:")
+        text_next = self.tr("Next:")
+        text_rectangle = self.tr("Rectangle:")
+        text_polygon = self.tr("Polygon:")
         return (
-            f"<b>Mode:</b> {self.canvas.get_mode()} - <b>Shortcuts:</b>"
-            " Previous: <b>A</b>, Next: <b>D</b>, Rectangle: <b>R</b>,"
-            " Polygon: <b>P</b>"
+            f"<b>{text_mode}</b> {self.canvas.get_mode()} - <b>{text_shortcuts}</b>"
+            f" {text_previous} <b>A</b>, {text_next} <b>D</b>,"
+            f" {text_rectangle} <b>R</b>,"
+            f" {text_polygon}: <b>P</b>"
         )
 
     @pyqtSlot()
@@ -1028,7 +1125,7 @@ class LabelmeWidget(LabelDialog):
         toolbar.setObjectName(f"{title}ToolBar")
         toolbar.setOrientation(Qt.Vertical)
         toolbar.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonIconOnly)
-        toolbar.setIconSize(QtCore.QSize(20, 20))
+        toolbar.setIconSize(QtCore.QSize(24, 24))
         toolbar.setMaximumWidth(40)
         if actions:
             utils.add_actions(toolbar, actions)
@@ -1140,6 +1237,10 @@ class LabelmeWidget(LabelDialog):
         self.load_shapes(self.canvas.shapes)
         self.actions.undo.setEnabled(self.canvas.is_shape_restorable)
 
+    def documentation(self):
+        url = "https://anylabeling.com/"  # NOQA
+        webbrowser.open(url)
+
     def contact(self):
         url = "https://aicurious.io/contact/"  # NOQA
         webbrowser.open(url)
@@ -1166,6 +1267,8 @@ class LabelmeWidget(LabelDialog):
             self.clear_auto_labeling_marks()
             self.auto_labeling_widget.set_auto_labeling_mode(None)
 
+        self.set_text_editing(False)
+
         self.canvas.set_editing(edit)
         self.canvas.create_mode = create_mode
         if edit:
@@ -1221,6 +1324,7 @@ class LabelmeWidget(LabelDialog):
             else:
                 raise ValueError(f"Unsupported create_mode: {create_mode}")
         self.actions.edit_mode.setEnabled(not edit)
+        self.label_instruction.setText(self.get_labeling_instruction())
 
     def set_edit_mode(self):
         # Disable auto labeling
@@ -1228,6 +1332,8 @@ class LabelmeWidget(LabelDialog):
         self.auto_labeling_widget.set_auto_labeling_mode(None)
 
         self.toggle_draw_mode(True)
+        self.set_text_editing(True)
+        self.label_instruction.setText(self.get_labeling_instruction())
 
     def update_file_menu(self):
         current = self.filename
@@ -1293,6 +1399,9 @@ class LabelmeWidget(LabelDialog):
         shape.flags = flags
         shape.group_id = group_id
 
+        # Add to label history
+        self.label_dialog.add_label_history(shape.label)
+
         # Update unique label list
         if not self.unique_label_list.find_items_by_label(shape.label):
             unique_label_item = self.unique_label_list.create_item_from_label(
@@ -1356,18 +1465,7 @@ class LabelmeWidget(LabelDialog):
         self.actions.duplicate.setEnabled(n_selected)
         self.actions.copy.setEnabled(n_selected)
         self.actions.edit.setEnabled(n_selected == 1)
-        if len(selected_shapes) == 1:
-            self.shape_text_label.setText("Object Text")
-            self.shape_text_edit.textChanged.disconnect()
-            self.shape_text_edit.setPlainText(selected_shapes[0].text)
-            self.shape_text_edit.textChanged.connect(self.shape_text_changed)
-        else:
-            self.shape_text_label.setText("Image Text")
-            self.shape_text_edit.textChanged.disconnect()
-            self.shape_text_edit.setPlainText(
-                self.other_data.get("image_text", "")
-            )
-            self.shape_text_edit.textChanged.connect(self.shape_text_changed)
+        self.set_text_editing(True)
 
     def add_label(self, shape):
         if shape.group_id is None:
@@ -1626,10 +1724,14 @@ class LabelmeWidget(LabelDialog):
             or not text
             or self.canvas.shapes[-1].label == AutoLabelingMode.OBJECT
         ):
-            previous_text = self.label_dialog.edit.text()
-            text, flags, group_id = self.label_dialog.pop_up(text)
-            if not text:
-                self.label_dialog.edit.setText(previous_text)
+            last_label = self.find_last_label()
+            if self._config["auto_use_last_label"] and last_label:
+                text = last_label
+            else:
+                previous_text = self.label_dialog.edit.text()
+                text, flags, group_id = self.label_dialog.pop_up(text)
+                if not text:
+                    self.label_dialog.edit.setText(previous_text)
 
         if text and not self.validate_label(text):
             self.error_message(
@@ -1662,7 +1764,7 @@ class LabelmeWidget(LabelDialog):
         self.set_scroll(orientation, value)
 
     def set_scroll(self, orientation, value):
-        self.scroll_bars[orientation].setValue(value)
+        self.scroll_bars[orientation].setValue(round(value))
         self.scroll_values[orientation][self.filename] = value
 
     def set_zoom(self, value):
@@ -1691,8 +1793,8 @@ class LabelmeWidget(LabelDialog):
         if canvas_width_old != canvas_width_new:
             canvas_scale_factor = canvas_width_new / canvas_width_old
 
-            x_shift = round(pos.x() * canvas_scale_factor) - pos.x()
-            y_shift = round(pos.y() * canvas_scale_factor) - pos.y()
+            x_shift = round(pos.x() * canvas_scale_factor - pos.x())
+            y_shift = round(pos.y() * canvas_scale_factor - pos.y())
 
             self.set_scroll(
                 Qt.Horizontal,
@@ -1718,21 +1820,25 @@ class LabelmeWidget(LabelDialog):
     def enable_keep_prev_scale(self, enabled):
         self._config["keep_prev_scale"] = enabled
         self.actions.keep_prev_scale.setChecked(enabled)
+        save_config(self._config)
 
     def enable_show_cross_line(self, enabled):
         self._config["show_cross_line"] = enabled
         self.actions.show_cross_line.setChecked(enabled)
         self.canvas.set_show_cross_line(enabled)
+        save_config(self._config)
 
     def enable_show_groups(self, enabled):
         self._config["show_groups"] = enabled
         self.actions.show_groups.setChecked(enabled)
         self.canvas.set_show_groups(enabled)
+        save_config(self._config)
 
     def enable_show_texts(self, enabled):
         self._config["show_texts"] = enabled
         self.actions.show_texts.setChecked(enabled)
         self.canvas.set_show_texts(enabled)
+        save_config(self._config)
 
     def on_new_brightness_contrast(self, qimage):
         self.canvas.load_pixmap(
@@ -1762,10 +1868,43 @@ class LabelmeWidget(LabelDialog):
         for item in self.label_list:
             item.setCheckState(Qt.Checked if value else Qt.Unchecked)
 
+    def get_next_files(self, filename, num_files):
+        """Get the next files in the list."""
+        if not self.image_list:
+            return []
+        filenames = []
+        current_index = 0
+        if filename is not None:
+            try:
+                current_index = self.image_list.index(filename)
+            except ValueError:
+                return []
+            filenames.append(filename)
+        for _ in range(num_files):
+            if current_index + 1 < len(self.image_list):
+                filenames.append(self.image_list[current_index + 1])
+                current_index += 1
+            else:
+                filenames.append(self.image_list[-1])
+                break
+        return filenames
+
+    def inform_next_files(self, filename):
+        """Inform the next files to be annotated.
+        This list can be used by the user to preload the next files
+        or running a background process to process them
+        """
+        next_files = self.get_next_files(filename, 5)
+        if next_files:
+            self.next_files_changed.emit(next_files)
+
     def load_file(self, filename=None):  # noqa: C901
         """Load the specified file, or the last opened file if None."""
 
+        # For auto labeling, clear the previous marks
+        # and inform the next files to be annotated
         self.clear_auto_labeling_marks()
+        self.inform_next_files(filename)
 
         # Changing file_list_widget loads file
         if filename in self.image_list and (
@@ -1994,6 +2133,7 @@ class LabelmeWidget(LabelDialog):
             Qt.ControlModifier | Qt.ShiftModifier
         ):
             self._config["keep_prev"] = True
+            save_config(self._config)
 
         if not self.may_continue():
             return
@@ -2011,6 +2151,7 @@ class LabelmeWidget(LabelDialog):
                 self.load_file(filename)
 
         self._config["keep_prev"] = keep_prev
+        save_config(self._config)
 
     def open_next_image(self, _value=False, load=True):
         keep_prev = self._config["keep_prev"]
@@ -2018,6 +2159,7 @@ class LabelmeWidget(LabelDialog):
             Qt.ControlModifier | Qt.ShiftModifier
         ):
             self._config["keep_prev"] = True
+            save_config(self._config)
 
         if not self.may_continue():
             return
@@ -2040,6 +2182,7 @@ class LabelmeWidget(LabelDialog):
             self.load_file(self.filename)
 
         self._config["keep_prev"] = keep_prev
+        save_config(self._config)
 
     def open_file(self, _value=False):
         if not self.may_continue():
@@ -2244,6 +2387,13 @@ class LabelmeWidget(LabelDialog):
 
     def toggle_keep_prev_mode(self):
         self._config["keep_prev"] = not self._config["keep_prev"]
+        save_config(self._config)
+
+    def toggle_auto_use_last_label(self):
+        self._config["auto_use_last_label"] = not self._config[
+            "auto_use_last_label"
+        ]
+        save_config(self._config)
 
     def remove_selected_point(self):
         self.canvas.remove_selected_point()
@@ -2460,6 +2610,19 @@ class LabelmeWidget(LabelDialog):
         Find the last label in the label list.
         Exclude labels for auto labeling.
         """
+
+        # Get from dialog history
+        last_label = self.label_dialog.get_last_label()
+        if last_label:
+            return last_label
+
+        # Get selected label from the label list
+        items = self.label_list.selected_items()
+        if items:
+            shape = items[0].data(Qt.UserRole)
+            return shape.label
+
+        # Get the last label from the label list
         for item in reversed(self.label_list):
             shape = item.data(Qt.UserRole)
             if shape.label not in [
@@ -2468,6 +2631,8 @@ class LabelmeWidget(LabelDialog):
                 AutoLabelingMode.REMOVE,
             ]:
                 return shape.label
+
+        # No label is found
         return ""
 
     def finish_auto_labeling_object(self):
@@ -2483,14 +2648,20 @@ class LabelmeWidget(LabelDialog):
             return
 
         # Ask a label for the object
-        previous_text = self.label_dialog.edit.text()
-        text, flags, group_id = self.label_dialog.pop_up(
-            text=self.find_last_label(),
-            flags={},
-            group_id=None,
-        )
-        if not text:
-            self.label_dialog.edit.setText(previous_text)
+        text, flags, group_id = "", {}, None
+        last_label = self.find_last_label()
+        if self._config["auto_use_last_label"] and last_label:
+            text = last_label
+        else:
+            previous_text = self.label_dialog.edit.text()
+            text, flags, group_id = self.label_dialog.pop_up(
+                text=self.find_last_label(),
+                flags={},
+                group_id=None,
+            )
+            if not text:
+                self.label_dialog.edit.setText(previous_text)
+                return
 
         if not self.validate_label(text):
             self.error_message(
@@ -2501,6 +2672,9 @@ class LabelmeWidget(LabelDialog):
             )
             return
 
+        # Add to label history
+        self.label_dialog.add_label_history(text)
+
         # Update label for the object
         updated_shapes = False
         for shape in self.canvas.shapes:
@@ -2523,6 +2697,7 @@ class LabelmeWidget(LabelDialog):
                     )
 
                 # Update label list
+                self._update_shape_color(shape)
                 item = self.label_list.find_item_by_shape(shape)
                 if shape.group_id is None:
                     color = shape.fill_color.getRgb()[:3]
@@ -2543,3 +2718,35 @@ class LabelmeWidget(LabelDialog):
 
         if updated_shapes:
             self.set_dirty()
+
+    def set_text_editing(self, enable):
+        """Set text editing."""
+        if enable:
+            # Enable text editing and set shape text from selected shape
+            if len(self.canvas.selected_shapes) == 1:
+                self.shape_text_label.setText(self.tr("Object Text"))
+                self.shape_text_edit.textChanged.disconnect()
+                self.shape_text_edit.setPlainText(
+                    self.canvas.selected_shapes[0].text
+                )
+                self.shape_text_edit.textChanged.connect(
+                    self.shape_text_changed
+                )
+            else:
+                self.shape_text_label.setText(self.tr("Image Text"))
+                self.shape_text_edit.textChanged.disconnect()
+                self.shape_text_edit.setPlainText(
+                    self.other_data.get("image_text", "")
+                )
+                self.shape_text_edit.textChanged.connect(
+                    self.shape_text_changed
+                )
+            self.shape_text_edit.setDisabled(False)
+        else:
+            self.shape_text_edit.setDisabled(True)
+            self.shape_text_label.setText(
+                self.tr("Switch to Edit mode for text editing")
+            )
+            self.shape_text_edit.textChanged.disconnect()
+            self.shape_text_edit.setPlainText("")
+            self.shape_text_edit.textChanged.connect(self.shape_text_changed)
