@@ -2,18 +2,33 @@
 
 from PyQt5.QtWidgets import QVBoxLayout, QWidget
 
-from .label_widget import LabelmeWidget
+from .label_widget import LabelingWidget
 
 
 class LabelingWrapper(QWidget):
     """Wrapper widget for labeling module"""
 
-    def __init__(self, parent):
+    def __init__(
+        self,
+        parent,
+        config=None,
+        filename=None,
+        output=None,
+        output_file=None,
+        output_dir=None,
+    ):
         super().__init__()
         self.parent = parent
 
         # Create a labeling widget
-        view = LabelmeWidget(self)
+        view = LabelingWidget(
+            self,
+            config=config,
+            filename=filename,
+            output=output,
+            output_file=output_file,
+            output_dir=output_dir,
+        )
 
         # Create the main layout and put labeling into
         main_layout = QVBoxLayout()
