@@ -68,7 +68,7 @@ class Canvas(
         self.line = Shape()
         self.prev_point = QtCore.QPoint()
         self.prev_move_point = QtCore.QPoint()
-        self.offsets = QtCore.QPoint(), QtCore.QPoint()
+        self.offsets = QtCore.QPointF(), QtCore.QPointF()
         self.scale = 1.0
         self.pixmap = QtGui.QPixmap()
         self.visible = {}
@@ -96,6 +96,17 @@ class Canvas(
         self.show_shape_groups = True
         self.show_texts = True
 
+        self.is_loading = False
+        self.loading_text = self.tr("Loading...")
+        self.loading_angle = 0
+
+    def set_loading(self, is_loading: bool, loading_text: str = None):
+        """Set loading state"""
+        self.is_loading = is_loading
+        if loading_text:
+            self.loading_text = loading_text
+        self.update()
+
     def set_auto_labeling_mode(self, mode: AutoLabelingMode):
         """Set auto labeling mode"""
         if mode == AutoLabelingMode.NONE:
@@ -209,14 +220,17 @@ class Canvas(
 
     def get_mode(self):
         """Get current mode"""
-        if self.is_auto_labeling:
-            return "Auto Labeling"
+        if (
+            self.is_auto_labeling
+            and self.auto_labeling_mode != AutoLabelingMode.NONE
+        ):
+            return self.tr("Auto Labeling")
         if self.mode == self.CREATE:
-            return "Drawing"
+            return self.tr("Drawing")
         elif self.mode == self.EDIT:
-            return "Editing"
+            return self.tr("Editing")
         else:
-            return "Unknown"
+            return self.tr("Unknown")
 
     def set_editing(self, value=True):
         """Set editing mode. Editing is set to False, user is drawing"""
@@ -246,6 +260,8 @@ class Canvas(
     # QT Overload
     def mouseMoveEvent(self, ev):  # noqa: C901
         """Update line with last point and current coordinates"""
+        if self.is_loading:
+            return
         try:
             pos = self.transform_pos(ev.localPos())
         except AttributeError:
@@ -406,6 +422,8 @@ class Canvas(
     # QT Overload
     def mousePressEvent(self, ev):
         """Mouse press event"""
+        if self.is_loading:
+            return
         pos = self.transform_pos(ev.localPos())
         if ev.button() == QtCore.Qt.LeftButton:
             if self.drawing():
@@ -469,6 +487,8 @@ class Canvas(
     # QT Overload
     def mouseReleaseEvent(self, ev):
         """Mouse release event"""
+        if self.is_loading:
+            return
         if ev.button() == QtCore.Qt.RightButton:
             menu = self.menus[len(self.selected_shapes_copy) > 0]
             self.restore_cursor()
@@ -538,6 +558,8 @@ class Canvas(
     # QT Overload
     def mouseDoubleClickEvent(self, _):
         """Mouse double click event"""
+        if self.is_loading:
+            return
         # We need at least 4 points here, since the mousePress handler
         # adds an extra one before this handler is called.
         if (
@@ -598,7 +620,7 @@ class Canvas(
         y1 = top - point.y()
         x2 = right - point.x()
         y2 = bottom - point.y()
-        self.offsets = QtCore.QPoint(x1, y1), QtCore.QPoint(x2, y2)
+        self.offsets = QtCore.QPointF(x1, y1), QtCore.QPointF(x2, y2)
 
     def bounded_move_vertex(self, pos):
         """Move a vertex. Adjust position to be bounded by pixmap border"""
@@ -681,8 +703,8 @@ class Canvas(
         # Try to move in one direction, and if it fails in another.
         # Give up if both fail.
         point = shapes[0][0]
-        offset = QtCore.QPoint(2.0, 2.0)
-        self.offsets = QtCore.QPoint(), QtCore.QPoint()
+        offset = QtCore.QPointF(2.0, 2.0)
+        self.offsets = QtCore.QPointF(), QtCore.QPointF()
         self.prev_point = point
         if not self.bounded_move_shapes(shapes, point - offset):
             self.bounded_move_shapes(shapes, point + offset)
@@ -709,6 +731,38 @@ class Canvas(
         p.drawPixmap(0, 0, self.pixmap)
         Shape.scale = self.scale
 
+        # Draw loading/waiting screen
+        if self.is_loading:
+            # Draw a semi-transparent rectangle
+            p.setPen(Qt.NoPen)
+            p.setBrush(QtGui.QColor(0, 0, 0, 100))
+            p.drawRect(self.pixmap.rect())
+
+            # Draw a spinning wheel
+            p.setPen(QtGui.QColor(255, 255, 255))
+            p.setBrush(Qt.NoBrush)
+            p.save()
+            p.translate(self.pixmap.width() / 2, self.pixmap.height() / 2 - 50)
+            p.rotate(self.loading_angle)
+            p.drawEllipse(-20, -20, 40, 40)
+            p.drawLine(0, 0, 0, -20)
+            p.restore()
+            self.loading_angle += 30
+            if self.loading_angle >= 360:
+                self.loading_angle = 0
+
+            # Draw the loading text
+            p.setPen(QtGui.QColor(255, 255, 255))
+            p.setFont(QtGui.QFont("Arial", 20))
+            p.drawText(
+                self.pixmap.rect(),
+                Qt.AlignCenter,
+                self.loading_text,
+            )
+            p.end()
+            self.update()
+            return
+
         # Draw groups
         if self.show_shape_groups:
             pen = QtGui.QPen(QtGui.QColor("#AAAAAA"), 2, Qt.SolidLine)
@@ -833,12 +887,12 @@ class Canvas(
             p.setPen(pen)
             p.setOpacity(0.5)
             p.drawLine(
-                QtCore.QPoint(self.prev_move_point.x(), 0),
-                QtCore.QPoint(self.prev_move_point.x(), self.pixmap.height()),
+                QtCore.QPointF(self.prev_move_point.x(), 0),
+                QtCore.QPointF(self.prev_move_point.x(), self.pixmap.height()),
             )
             p.drawLine(
-                QtCore.QPoint(0, self.prev_move_point.y()),
-                QtCore.QPoint(self.pixmap.width(), self.prev_move_point.y()),
+                QtCore.QPointF(0, self.prev_move_point.y()),
+                QtCore.QPointF(self.pixmap.width(), self.prev_move_point.y()),
             )
 
         p.end()
@@ -850,14 +904,14 @@ class Canvas(
     def offset_to_center(self):
         """Calculate offset to the center"""
         if self.pixmap is None:
-            return QtCore.QPoint()
+            return QtCore.QPointF()
         s = self.scale
         area = super().size()
         w, h = self.pixmap.width() * s, self.pixmap.height() * s
         area_width, area_height = area.width(), area.height()
         x = (area_width - w) / (2 * s) if area_width > w else 0
         y = (area_height - h) / (2 * s) if area_height > h else 0
-        return QtCore.QPoint(x, y)
+        return QtCore.QPointF(x, y)
 
     def out_off_pixmap(self, p):
         """Check if a position is out of pixmap"""
@@ -1013,8 +1067,8 @@ class Canvas(
             if 0 <= ua <= 1 and 0 <= ub <= 1:
                 x = x1 + ua * (x2 - x1)
                 y = y1 + ua * (y2 - y1)
-                m = QtCore.QPoint((x3 + x4) / 2, (y3 + y4) / 2)
-                d = utils.distance(m - QtCore.QPoint(x2, y2))
+                m = QtCore.QPointF((x3 + x4) / 2, (y3 + y4) / 2)
+                d = utils.distance(m - QtCore.QPointF(x2, y2))
                 yield d, i, (x, y)
 
     # These two, along with a call to adjustSize are required for the
@@ -1071,13 +1125,13 @@ class Canvas(
                 self.snapping = False
         elif self.editing():
             if key == QtCore.Qt.Key_Up:
-                self.move_by_keyboard(QtCore.QPoint(0.0, -MOVE_SPEED))
+                self.move_by_keyboard(QtCore.QPointF(0.0, -MOVE_SPEED))
             elif key == QtCore.Qt.Key_Down:
-                self.move_by_keyboard(QtCore.QPoint(0.0, MOVE_SPEED))
+                self.move_by_keyboard(QtCore.QPointF(0.0, MOVE_SPEED))
             elif key == QtCore.Qt.Key_Left:
-                self.move_by_keyboard(QtCore.QPoint(-MOVE_SPEED, 0.0))
+                self.move_by_keyboard(QtCore.QPointF(-MOVE_SPEED, 0.0))
             elif key == QtCore.Qt.Key_Right:
-                self.move_by_keyboard(QtCore.QPoint(MOVE_SPEED, 0.0))
+                self.move_by_keyboard(QtCore.QPointF(MOVE_SPEED, 0.0))
 
     # QT Overload
     def keyReleaseEvent(self, ev):
